const OSS = require('ali-oss');
const { queryAuth } = require('./ossAuth');
const ProgressBar = require('progress');
const co = require('co');
const process = require('process');

exports.uploadFiles = async paths => {
  let data = await queryAuth();
  if (!data) {
    return;
  }

  let bucket = 'qgt-style';
  let region = 'oss-cn-hangzhou';
  const client = new OSS({
    region: region,
    accessKeyId: data.credentials.accessKeyId,
    accessKeySecret: data.credentials.accessKeySecret,
    stsToken: data.credentials.securityToken,
    bucket: bucket
  });

  let uploadInfo = [];
  for (let i = 0; i < paths.length; i++) {
    let path = paths[i];
    let info = await uploadCore(client, path, bucket, region);
    uploadInfo.push(info);
  }

  return uploadInfo;
};

async function uploadCore(client, path, bucket, region) {
  let tmps = path.split('/');
  let fileName = tmps[tmps.length - 1];
  try {
    var bar = new ProgressBar('Upload Img [:bar] :percent :etas', {
      total: 100
    });
    let uploadResult = await co(function*() {
      let checkPoint;
      let lastP = 0;
      for (var i = 0; i < 10; i++) {
        try {
          let result = yield client.multipartUpload(fileName, path, {
            checkPoint: checkPoint,
            progress: function*(p, cpt) {
              checkpoint = cpt;
              bar.tick(100 * (p - lastP));
              lastP = p;
            }
          });
          return Promise.resolve(result);
        } catch (error) {
          console.log('\nupload error: ', error, '上传失败，请重新运行');
          process.exit();
        }
      }
    });

    console.log(path);
    return {
      localPath: path,
      fileName,
      remoteUrl: `http://${bucket}.${region}.aliyuncs.com/${fileName}`
    };
  } catch (error) {
    console.error('\nupload error: ', error);
  }
}
