const { request, checkBaseResponse } = require("../util/request.js");
const process = require("process");
const utils = require("utility");
const { clearAuth } = require("./davaoAuth");

function parseAuth(createUploadVideoResponse) {
  let { uploadAddress, uploadAuth, videoId } = createUploadVideoResponse;
  let addressInfo = JSON.parse(utils.base64decode(uploadAddress));
  let authInfo = JSON.parse(utils.base64decode(uploadAuth));
  return { addressInfo, authInfo, videoId };
}

/**
 * 获取上传证书
 * @param {*} fileName 文件名字
 * @param {*} title 视频标题
 */
exports.queryAuth = async function(fileName, title) {
  console.info("-----> start fetch auth...");
  let response = await request("videoAuthService/getUploadAuth", {
    fileName,
    title,
    auth: process.env.DAVAO_AUTH
  });

  let { baseResponse, createUploadVideoResponse } = response;
  if (checkBaseResponse(baseResponse)) {
    console.info("<!--- fetch success!");
    return parseAuth(createUploadVideoResponse);
  } else {
    console.error("<!--- error: fetch error " + baseResponse.message);
    if (baseResponse.message === "auth error") {
      clearAuth();
    }
    return null;
  }
};
