const OSS = require('ali-oss');
const { queryAuth } = require('./vodAuth.js');
const co = require('co');
const ProgressBar = require('progress');
const fs = require('mz/fs');
const path = require('path');
const process = require('process');

const VIDEO_EXT = ['mov', 'mp4', 'avi', 'wmv', 'mkv', 'flv'];

/**
 * 校验文件
 * 1. 校验文件是否存在
 * 2. 校验文件是否为视频
 * @param {*} filePath
 */
validateFilePath = async function(filePath) {
  let exist = await fs.exists(filePath);
  if (!exist) {
    console.error(`error: video At : ${filePath} dose not exist!!`);
    return false;
  }

  let ext = path.extname(filePath).substring(1);
  if (-1 === VIDEO_EXT.indexOf(ext.toLowerCase())) {
    console.error(`error: video's ext [${ext}] is not supportted!!`);
    return false;
  }

  return true;
};

/**
 * oss 上传文件
 * @param {*} filePath 文件本地路径
 */
exports.vodUpload = async function(filePath) {
  if (!await validateFilePath(filePath)) {
    return null;
  }
  let { base, name } = path.parse(filePath);
  let authResponse = await queryAuth(base, name);
  if (!authResponse) {
    return;
  }
  let { addressInfo, authInfo, videoId } = authResponse;
  let info = {
    bucket: addressInfo.Bucket,
    region: 'oss-cn-shanghai',
    accessKeyId: authInfo.AccessKeyId,
    accessKeySecret: authInfo.AccessKeySecret,
    stsToken: authInfo.SecurityToken
  };

  let client = new OSS(info);
  try {
    var bar = new ProgressBar('Upload Progress [:bar] :percent :etas', {
      total: 100
    });
    let uploadResult = await co(function*() {
      let checkPoint;
      let lastP = 0;
      for (var i = 0; i < 5; i++) {
        try {
          let result = yield client.multipartUpload(
            addressInfo.FileName,
            filePath,
            {
              checkPoint: checkPoint,
              progress: function*(p, cpt) {
                checkpoint = cpt;
                bar.tick(100 * (p - lastP));
                lastP = p;
              }
            }
          );
          return Promise.resolve(result);
        } catch (error) {
          console.log('\nupload error: ', error, '启动断点续传');
        }
      }
    });
    console.info('\nUpload Success! VideoId: ' + videoId);
    return videoId;
  } catch (error) {
    console.error('\nupload error: ', error);
  }
  return null;
};
