const rp = require('request-promise');
const cheerio = require('cheerio');
const path = require('path');
const process = require('process');
const { mkdirs, removeDir } = require('../../util/io');
const fs = require('mz/fs');
const { uploadFiles } = require('../ossUpload');
const { request, checkBaseResponse } = require('../../util/request.js');
const qrcode = require('qrcode-terminal');

exports.convert = async function(configPath) {
  //读取config
  let exist = await fs.exists(configPath);
  if (!exist) {
    console.error('error: 配置文件不存在');
    return;
  }
  let config = JSON.parse(await fs.readFile(configPath, 'utf-8'));
  let path = config.path;
  let share = config.share;
  // path = 'https://mp.weixin.qq.com/s/frzgt0KXZUz_5YcaLHc2sw';
  //下载文件
  var options = {
    uri: path
  };
  let response = await rp(options);
  let $ = cheerio.load(response);

  //获取网页名称
  let token = new Date().getTime();

  //处理文件，获取核心展示区域内容
  let title = $('#activity-name')
    .text()
    .trim();
  let mainContent = $('#js_content').html();
  let { imgs, content } = await exportFile(mainContent, token);

  //上传图片
  console.info('step 3: 开始图片上传');
  let uploadResult = await uploadFiles(
    imgs.map(info => {
      return info.localPath;
    })
  );

  //构建替换文本;
  console.info('step 4: 开始进行文本内容处理');
  let replaceMap = {};
  imgs.map(img => {
    uploadResult.map(uploadImg => {
      if (img.localPath === uploadImg.localPath) {
        replaceMap[img.wxPath] = uploadImg.remoteUrl;
      }
    });
  });

  //替换文本中内容
  mainContent = mainContent.replace(/data-src="(.*?)"/g, (match, p1) => {
    return `data-src="${replaceMap[p1]}"`;
  });

  //把data-src写到src中
  $ = cheerio.load('');
  const body = $('body');
  body.html(mainContent);
  body
    .find('img')
    .toArray()
    .map(item => {
      $(item).attr('src', $(item).data('src'));
    });

  let tmpPath = getTmpsFolder(token);
  let destContent = body.html().trim();

  await fs.writeFile(`${tmpPath}/dest.html`, destContent);
  console.info('step 5: 数据处理完善，文件地址：' + `${tmpPath}/dest.html`);

  //对接api.qingguatang.com
  console.info('step 6: 开始新建一个文章到 bops 中');
  await addDocument(title, destContent, share);

  console.info('step 7: 删除临时文件夹');
  removeDir(tmpPath);
};

async function addDocument(title, html, share) {
  let data = {
    title,
    subject: title,
    questionType: 'document',
    tagId: '69241229652525056', //微信文章
    levelId: '1',
    questionDictionaryList: [
      {
        key: 'document',
        value: JSON.stringify({
          subject: html.trim(),
          subtitle: title,
          share
        })
      }
    ]
  };
  let response = await request('questionAPI/add', data);
  if (response) {
    let { question } = response;
    let questionId = question.id;
    qrcode.generate(`http://page.unreach.io/weixin/${questionId}`);
    console.info(`url: http://page.unreach.io/weixin/${questionId}`);
  }
}

/**
 * 下载文件到本地
 * @param {*} html
 * @param {*} token
 */
async function exportFile(html, token) {
  let tmpPath = getTmpsFolder(token);

  await mkdirs(tmpPath);

  //创建content.html 保留文件
  console.info('step 1: 开始爬去网站内容');
  let diskContentPath = `${tmpPath}/content.html`;
  await fs.writeFile(`${tmpPath}/content.html`, html);

  //下载图片
  console.info('step 2: 开始下载网站图片');
  let diskImgInfos = await exportImgs(html, tmpPath);
  console.log(diskImgInfos, diskImgInfos.length);

  return {
    imgs: diskImgInfos,
    content: diskContentPath
  };
}

/**
 * 下载图片到本地
 * @param {*} html
 * @param {*} tmpPath
 */
async function exportImgs(html, tmpPath) {
  let $ = cheerio.load(html);
  let imageSrcs = [];

  //获取所有的图片地址
  $('img')
    .toArray()
    .map(item => {
      imageSrcs.push($(item).data('src'));
    });

  let diskFiles = [];
  //下载所有的图片
  await Promise.all(
    imageSrcs.map(url => {
      let tmps = url.match(/wx_fmt=(.*)/);
      let suf = tmps[1];
      let fileName = url.split('/')[4];
      let localPath = `${tmpPath}/${fileName}.${suf}`;
      diskFiles.push({
        wxPath: url,
        fileName,
        localPath
      });
      return rp(url).pipe(fs.createWriteStream(localPath));
    })
  );

  return diskFiles;
}

function getTmpsFolder(token) {
  return path.join(
    process.env[process.platform == 'win32' ? 'USERPROFILE' : 'HOME'],
    `davao_${token}`
  );
}
